////////////////////////////////////////////////////////////////////////////////
//
// Fracter - A simple Mandelbrot Set viewer.
//
// Copyright (C) 2010 Sarah Eris Horsley Caffee
//
// This file is part of Fracter.
//
// Fracter is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
//
// Fractal.cpp
// Fractal calculator virtual class implementation.  Actual fractals should 
// implement this.
//
////////////////////////////////////////////////////////////////////////////////

#include "Fractal.h"

////////////////////////////////////////////////////////////////////////////////
View::View(std::string t, 
	   double re, double im, double s, 
	   unsigned int max) :
   type (t),
   re_min (re),
   im_max (im),
   size (s),
   max_iter (max)
   {
   }

////////////////////////////////////////////////////////////////////////////////
View::~View()
   {
   }

////////////////////////////////////////////////////////////////////////////////
std::string const & View::get_type() const
   {
   return type;
   }

////////////////////////////////////////////////////////////////////////////////

Fractal::Fractal() :
   display_name ("No fractal"),
   fractal_name ("No fractal"),
   re_min (0),
   im_max (0),
   size (0),
   max_iter (0),
   calc_needed (false),
   surface (NULL)
   {
   }


////////////////////////////////////////////////////////////////////////////////
Fractal::~Fractal()
   {
   }


////////////////////////////////////////////////////////////////////////////////
bool Fractal::init(SDL_Surface * surf)
   {
   if (!surf)
      {
      return false;
      }
   else
      {
      surface = surf;
      }
   calc_needed = true;
   return true;
   }


////////////////////////////////////////////////////////////////////////////////
double Fractal::get_re_min() const
   {
   return re_min;
   }


////////////////////////////////////////////////////////////////////////////////
double Fractal::set_re_min(const double r_min)
   {
   double t = re_min;
   re_min = r_min;
   return t;
   }


////////////////////////////////////////////////////////////////////////////////
double Fractal::get_im_max() const
   {
   return im_max;
   }


////////////////////////////////////////////////////////////////////////////////
double Fractal::set_im_max(const double i_max)
   {
   double t = im_max;
   im_max = i_max;
   return t;
   }


////////////////////////////////////////////////////////////////////////////////
double Fractal::get_size() const
   {
   return size;
   }


////////////////////////////////////////////////////////////////////////////////
double Fractal::set_size(const double s)
   {
   double t = size;
   if (s > 0.0)
      size = s;
   else
      size = 1.0;
   return t;
   }


////////////////////////////////////////////////////////////////////////////////
unsigned int Fractal::get_max_iter() const
   {
   return max_iter;
   }


////////////////////////////////////////////////////////////////////////////////
unsigned int Fractal::set_max_iter(const unsigned int iter)
   {
   unsigned int t = max_iter;
   if (iter > 0)
      max_iter = iter;
   else
      max_iter = 1;
   return t;
   }


////////////////////////////////////////////////////////////////////////////////
unsigned int Fractal::inc_max_iter()
   {
   unsigned int t = max_iter;
   max_iter++;
   return t;
   }


////////////////////////////////////////////////////////////////////////////////
unsigned int Fractal::dec_max_iter()
   {
   unsigned int t = max_iter;
   if (max_iter > 0)
      max_iter--;
   return t;
   }


////////////////////////////////////////////////////////////////////////////////
void Fractal::set_calc_needed()
   {
   calc_needed = true;
   }


////////////////////////////////////////////////////////////////////////////////
std::string const & Fractal::get_display_name() const
   {
   return display_name;
   }


////////////////////////////////////////////////////////////////////////////////
std::string const & Fractal::get_fractal_name() const
   {
   return fractal_name;
   }


////////////////////////////////////////////////////////////////////////////////
bool Fractal::get_option(std::string option, void * value) const 
   {
   return false; 
   }

////////////////////////////////////////////////////////////////////////////////
bool Fractal::set_option(std::string option, void * value) 
   { 
   return false; 
   }

////////////////////////////////////////////////////////////////////////////////
View * Fractal::save_view()
   {
   return new View(typeid(*this).name(), re_min, im_max, size, max_iter);
   }

////////////////////////////////////////////////////////////////////////////////
void Fractal::restore_view(View * v)
   {
   fractal_name = v->type;
   re_min = v->re_min;
   im_max = v->im_max;
   size = v->size;
   max_iter = v->max_iter;
   calc_needed = true;
   }


////////////////////////////////////////////////////////////////////////////////
void Fractal::draw_pixel(int x, int y, Uint32 * color)
   {
   static char * data;

   if (SDL_MUSTLOCK(surface)) SDL_LockSurface(surface);

   data = (char *) surface->pixels 
	 + y * surface->pitch
	 + x * surface->format->BytesPerPixel;
   memcpy(data, color, surface->format->BytesPerPixel);

   if (SDL_MUSTLOCK(surface)) SDL_UnlockSurface(surface);
   }


////////////////////////////////////////////////////////////////////////////////
FractalFactory::~FractalFactory()
   {
   get_Fractal_map().clear();
   }


////////////////////////////////////////////////////////////////////////////////

std::map <std::string, FractalFactory *> & get_Fractal_map()
   {
   static std::map <std::string, FractalFactory *> the_Fractal_map;
   return the_Fractal_map;
   }

